
--required libraries
class = require 'scripts/libs/middleclass'
flux = require 'scripts/libs/flux'
stateful = require 'scripts/libs/stateful'
inspect = require 'scripts/libs/inspect'
cron = require 'scripts/libs/cron'
beholder = require 'scripts/libs/beholder'
require 'scripts/auxiliary'

--load our class/object scripts
function loadScripts()
	local names = {
		'scripts/grid'
	}
	for i,n in ipairs(names) do
		package.loaded[n] = nil
		require(n)
	end
end
loadScripts()

--Game class and initial state code
Game = class('Game'):include(stateful)
function Game:initialize() self:gotoState('Debug') end

Debug = Game:addState('Debug')

function Debug:enteredState()
	self.grid = Grid()
	
	self.obs = {}
	self.obs.clickGrid = beholder.observe('mousepressed', function(x,y,button)
		if button=='l' then
			self.grid:click(x,y,button)
		elseif button == 'r' then
			local t, r, c, match
			t, r, c = self.grid:clickedTile(x,y)
			if t then match = self.grid:getMatch(self.grid.tiles, r, c, true) end
			if match then flags[#flags+1] = inspect(match) end
		end
	end)
	self.obs.clearMatches = beholder.observe('keypressed', function(key, isrepeat)
		if key=='d' then
			self.grid:clearMatches()
			self.grid:dropTiles()
		end
	end)
	self.obs.shuffleGrid = beholder.observe('keypressed', function(key,isrepeat)
		if key == 'c' then
			self.grid = Grid()
			flags[1] = nil
		end
	end)
	self.obs.changeTile = beholder.observe('keypressed', function (key,isrepeat)
		local tile = self.grid:clickedTile(self.mx,self.my)
		if tile then
			if key=='1' then
				tile.color = 'r'
			elseif key=='2' then
				tile.color = 'g'
			elseif key=='3' then
				tile.color = 'b'
			elseif key=='4' then
				tile.color = 'y'
			elseif key=='5' then
				tile.color = 'p'
			elseif key=='6' then
				tile.color = 'k'
			end
		end
	end)
	self.obs.flagMatches = beholder.observe('keypressed', function (key,isrepeat)
		if key == 'a' then
			flags = {}
			local m = self.grid:getMatches()
			if m then
				for i = 1,#m do
					flags[i] = inspect(m[i])
				end
				local sum = 0
				for _,r in ipairs(self.grid.tiles) do
					for _,c in ipairs(r) do
						sum = sum + 1
					end
				end
				flags[1] = flags[1] .. " ..  # of tiles = " .. sum
			else
				flags[1] = 'nil'
			end
		elseif key == 'b' then
			flags = {}
		end
	end)
	
	
	
end

function Debug:exitedState()
	self.grid = nil
end

function Debug:update(dt)
	mx,my = lmo.getPosition()
	self.grid:update(dt)
end

function Debug:draw()
	self.grid:draw()
end

--love functions
function love.load()
	hotswapScriptsObs = beholder.observe('mousepressed', function(x,y,button) if button=='m' then loadScripts() end end)
	quitOnEsc = beholder.observe('keypressed', function(key,isrepeat) if key == 'escape' then love.event.quit() end end)
	game = Game()
	flags = {}
end

function love.update(dt)
	if not lk.isDown(' ') then
		game:update(dt)
	end
end

function love.draw()
	game:draw()
	for i,f in ipairs(flags) do lg.regPrint(tostring(f), 20, 20*i) end
end

--love callbacks
function love.gamepadpressed(joystick,button) beholder.trigger('gamepadpressed', joystick, button) end
function love.gamepadreleased(joystick,button) beholder.trigger('gamepadreleased', joystick, button) end
function love.keypressed(key,isrepeat) beholder.trigger('keypressed', key, isrepeat) end
function love.keyreleased(key) beholder.trigger('keyreleased', key) end
function love.textinput(text) beholder.trigger('textinput', text) end
function love.mousepressed(x,y,button) beholder.trigger('mousepressed', x, y, button) end
function love.mousereleased(x,y,button) beholder.trigger('mousereleased', x, y, button) end
function love.resize(w,h) beholder.trigger('resize', w, h) end
function love.quit() beholder.trigger('quit') end










