
--local tables/functions etc
local tileColors = {'r','g','b','y','p','k'}

local colorByLetter = {
	r = {255,0,0,100},
	g = {0,255,0,100},
	b = {0,0,255,100},
	y = {255,255,0,100},
	p = {255,0,255,100},
	k = {255,255,255,100}
}

local function log(txt)
	if debug then lfs.append('debug.log', txt .. '\r') end
end

--class declaration
Grid = class('Grid')

function Grid:initialize(xv,yv)
	self.x,self.y = xv or sw/2 - 256, yv or sh/2-256
	self.obs = {}
	self.clocks = {}
	self.obs.clearAfterClear = beholder.observe('SETTLED', function()
		local id = #self.clocks + 1
		function f(self) self.clearMatches(self) self.clocks[id] = nil end
		self.clocks[id] = cron.after(0.4, f, self)
	end)
	self.tweens = {}
	--assume 8x8
	
	--establish data structure
	self.tiles = {}
	for r = 1, 8 do
		local tbl = {}
		for c = 1,8 do
			local tile = {color = tileColors[lm.random(1,6)], x = self.x + 64*c-32, y = self.y + 64*r-32 }
			table.insert(tbl,tile)
		end
		table.insert(self.tiles, tbl)
	end
	self.newTiles = {}
	self.dropping = flux.group()
	self.highlight = {} --tiles that we're going to draw brighter
	log('grid created')
end

function Grid:update(dt)
	-- if self.clicked then flags[2] = inspect(self.clicked) end
	self.dropping:update(dt)
	for i,cl in ipairs(self.clocks) do
		cl:update(dt)
	end
	flux.update(dt)
	
	if self.clicked then
		local sr,sc = self.clicked[1], self.clicked[2]
		local coords = {
			{self.x + (sc-2)*64, self.y + (sr-1)*64, 64, 64}, --left
			{self.x + (sc)*64, self.y + (sr-1)*64, 64, 64}, --right
			{self.x + (sc-1)*64, self.y + (sr)*64, 64, 64}, --up
			{self.x + (sc-1)*64, self.y + (sr-2)*64, 64, 64}  --down
		}
		for i,xy in ipairs(coords) do
			if xy[1] > self.x and xy[1] < self.x+512 and xy[2] > self.y and xy[2] < self.y+512 then
				if lmo.isDown('l') and coll.pointBox(mx,my,xy) then
					if i == 1 then
						self:swapTiles(sr,sc,sr,sc-1)
						self:highlightTile(self.tiles[sr][sc-1])
					elseif i == 2 then
						self:swapTiles(sr,sc,sr,sc+1)
						self:highlightTile(self.tiles[sr][sc+1])
					elseif i == 3 then
						self:swapTiles(sr+1,sc,sr,sc)
						self:highlightTile(self.tiles[sr+1][sc])
					else
						self:swapTiles(sr-1,sc,sr,sc)
						self:highlightTile(self.tiles[sr-1][sc])
					end
					self.clicked = nil
					self:unhighlight()
				end
			end
		end
	end
	
end

function Grid:printTiles()-- for debugging mostly
	local str = ""
	for i,r in ipairs(self.tiles) do
		str = str .. '{'
		for j = 1,8 do
			if r[j] then
				str = str .. '\t' .. r[j].color
			else
				str = str .. '\tnil'
			end
		end
		str = str .. '\t} \n'
	end
	return str
end

function Grid:draw()
	lg.setStencil(function()
		lg.rectangle('fill',self.x,self.y,516,516)
	end)
		lg.setColor(white)
		local w = 2
		lg.setLineWidth(w)
		for i = 0,8 do
			lg.line(self.x + i*64+w, self.y + w, self.x + i*64+w, self.y + 512+w)
			lg.line(self.x + w, self.y + i*64+w, self.x + w+512, self.y + i*64+w)
		end
		for r = 1, 8 do
			for c = 1, 8 do
				if self.tiles[r][c] then self:drawTile(self.tiles[r][c]) end
				-- lg.setColor(white)
				-- lg.print( r .. ' ' .. c, x+64*c, y+64*r)
			end
		end
		for i,t in pairs(self.newTiles) do
			if t then self:drawTile(t) end
		end
	lg.setStencil()
	-- lg.setColor(255,0,128)
	-- if self.mx then
		-- lg.circle('fill', self.mx, self.my, 12, 24)
	-- end
	-- lg.setColor(255,255,255)
	-- lg.print(inspect(self.clicked),20,200)
	-- lg.print(#self.clocks, 20, 300)
	-- lg.print(inspect(self.highlight),20,400)
end

function Grid:drawTile(tile)
	if tile.alpha then
		local clrTbl = table.deepcopy(colorByLetter[tile.color])
		clrTbl[4] = tile.alpha
		lg.setColor(clrTbl)
	else
		lg.setColor(colorByLetter[tile.color])
	end
	lg.circle('fill', tile.x, tile.y, 24, 12)
end

function Grid:getMatch(tilesTable, r, c, vert)
	local x,y
	if vert then x = 0 y = 1 else x = 1 y = 0 end
	local this, checked, good = tilesTable[r][c], 0, true
	local match
	
	while good and x*c + y*r + checked < 8 do
		checked = checked + 1
		if tilesTable[r + y*checked][c + x*checked] == nil or this==nil then error('too fast') end
		if this.color ~= tilesTable[r + y*checked][c + x*checked].color then
			good = false
			checked = checked - 1
		end
	end
	if checked >= 2 then
		match = {}
		table.insert(match,{r,c})
		for i = 1, checked do
			table.insert(match, {r + y*i ,c + x*i})
		end
	end
	
	return match --returns table full of tile coords or nil
end

function Grid:getMatches()
	-- local c = 1
	local matches = {}
	local skipRows = {0,0,0,0,0,0,0,0}
	for r = 1, 8 do
		local skipCols = 0
		for c = 1,8 do
			if self.tiles[r][c] == nil then return nil end
			if skipRows[c] >= 1 then skipRows[c] = skipRows[c] - 1 end
			local m
			if skipCols >= 1 or c > 6 then
				skipCols = skipCols - 1
			else
				m = self:getMatch(self.tiles,r,c,false)
				if m then
					matches[#matches+1] = m
					skipCols = #m-1
				end
			end
			if skipRows[c] == 0 and r <= 6 then
				m = self:getMatch(self.tiles,r,c,true)
				if m then
					matches[#matches+1] = m
					skipRows[c] = #m-1
				end
			end
		end
	end
	if next(matches)~=nil then
		return matches
	end
end

function Grid:clearMatches()
	-- if not self.clearing then
		-- self.clearing = true
		local matches = self:getMatches()
		if matches then
			for i,match in ipairs(self:getMatches()) do
				for j,coords in ipairs(match) do
					self.tiles[coords[1]][coords[2]] = nil
				end
			end
			beholder:trigger('MATCHES CLEARED', matches)
			self:dropTiles()
		end
	-- end
end

function Grid:dropTiles()
	local ACCEL = 64
	
	local function getTileAbove(r,c)
		for i = r-1, 1, -1 do
			if self.tiles[i][c]~=nil then return i end
		end
		return nil
	end
	
	for c = 1,8 do
		local blanks = 0
		--shift remaining tiles down until all blanks are at the top
		for r = 8,1, -1 do
			local tile = self.tiles[r][c]
			if tile==nil then
				local up = getTileAbove(r,c)
				if up then
					self.tiles[r][c] = self.tiles[up][c]
					local duration = math.sqrt(2*(r-up)/ACCEL)
					self.dropping:to(self.tiles[r][c], duration, {y = self.tiles[r][c].y + 64*(r-up)}):ease('quadin'):oncomplete(function() if #self.dropping == 0 then beholder.trigger('SETTLED') end end)
					self.tiles[up][c] = nil
				else
					blanks = r
					break
				end
			end
		end
		
		for i = 1, blanks do
			--create tile and tween it down
			self.tiles[i][c] = {color = tileColors[lm.random(1,6)], x = self.x + 64*c-32, y = self.y + 32-64*(blanks-i+1) }
			local duration = math.sqrt(2*blanks/ACCEL)
			self.dropping:to(self.tiles[i][c], duration, {y = self.tiles[i][c].y + 64*blanks}):ease('quadin'):oncomplete(function() if #self.dropping == 0 then beholder.trigger('SETTLED') end end)
		end
	end
end

function Grid:clickedTile()
	local r,c
	if mx > self.x and mx < self.x + 512 and my > self.y and my < self.y + 512 then
		r = math.ceil((my-self.y)/64)
		c = math.ceil((mx-self.x)/64)
		if r < 1 or r > 8 or c < 1 or c > 8 then error (r .. ' ' .. c .. ' ' .. mx - self.x .. ' ' .. my - self.y) end
	else
		return nil
	end
	return self.tiles[r][c], r, c
end

function Grid:click(mx,my,button)
	if button=='l' and #self.clocks == 0 and #self.dropping == 0 then
		local tile, row, col = self:clickedTile(mx,my)
		if tile then
			log('clicked on r, c = ' .. row .. ', ' .. col)
			if self.clicked then
				if self.clicked[1] == row and self.clicked[2] == col then
					self.clicked = nil
					self:unhighlight()
				else
					self:highlightTile(tile)
					self:swapTiles(row, col, self.clicked[1], self.clicked[2])
					self.clicked = nil
					self:unhighlight()
				end
			else
				if not table.find(self.highlight, tile) then
					log('self.clicked set to {' .. row .. ', ' .. col .. '}')
					self.clicked = {row, col}
					self:highlightTile(tile)
				end
			end
		else
			self.clicked = nil
			self:unhighlight()
		end
	end
end

function Grid:swapTiles(ra, ca, rb, cb)
	local function inputsValid()
		if ra >= 1 and ra <= 8 and ca >= 1 and ca <= 8 and rb >= 1 and rb <= 8 and cb >= 1 and cb <= 8 then
			log('inputs of ' .. ra .. ' ' .. ca .. ' ' .. rb .. ' ' .. cb .. ' are valid swap') 
			return true
		else
			log('inputs of ' .. ra .. ' ' .. ca .. ' ' .. rb .. ' ' .. cb .. ' are invalid swap; deleting self.clicked') 
			if self.clicked then self.clicked = nil end
			return false
		end
	end
	
	if inputsValid() then
		if (math.abs(rb-ra) == 1 and (cb-ca)==0) or ((rb-ra) == 0 and math.abs(cb-ca) == 1) then
			log('tiles found to be adjacent, proceeding with swap')
			self.tiles[ra][ca], self.tiles[rb][cb] = self.tiles[rb][cb], self.tiles[ra][ca]
			local ta, tb = self.tiles[ra][ca], self.tiles[rb][cb]
			flux.to(ta, 0.3, {x = tb.x, y = tb.y}):ease('backout')
			flux.to(tb, 0.3, {x = ta.x, y = ta.y}):ease('backout')
			
			local id = #self.clocks + 1
			function f(self) self.clearMatches(self) self.clocks[id] = nil end
			self.clocks[id] = cron.after(0.4, f, self)
			self:unhighlight()
		else
			log('tiles not adjacent')
		end
	end
end

function Grid:unhighlight(tileTable)
	if tileTable then
		if tileTable.color then error("Don't put tiles in Grid:unhighlight() function.") end
		for i,tile in ipairs(tileTable) do
			--fade tile alpha back to 100
			flux.to(tile, 0.5, {alpha = 100}):ease('cubicin'):oncomplete(function() table.remove(self.highlight, table.find(self.highlight, tile)) end)
		end
	else
		for i,tile in ipairs(self.highlight) do
			--fade tile alpha back to 100
			flux.to(tile, 0.5, {alpha = 100}):ease('cubicin'):oncomplete(function() table.remove(self.highlight, table.find(self.highlight, tile)) end)
		end
	end
end

function Grid:highlightTile(v)
	if v then
		if table.find(self.highlight, v) then return nil end
	else
		return nil
	end
	if v.color then	--just one tile
		v.alpha = 255
		self.highlight[#self.highlight+1] = v
	else
		for i,tile in ipairs(v) do
			tile.alpha = 255
			self.highlight[#self.highlight+1] = tile
		end
	end
end






































